﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: Phong.frag 537 2012-04-04 15:35:42Z cr333 $
*/
#version 120

uniform float ambient;
uniform float diffuse;
uniform float specular;
uniform float shininess;

// the following are all in viewing coordinates
in vec3 N; // surface normal
in vec3 v; // vertex position
in vec3 l; // light position

in float valid;

void main (void)
{
	//if(N.z < 0.15) discard;

	// discard fragments inside saturated triangles
	if(valid < 0.99) discard;
	
	// compute vectors to light, viewer and reflected ray (all normalised)
	vec3 L = normalize(l);
	vec3 V = normalize(-v); // we are in view coordinates, so the eye is at (0,0,0)
	vec3 R = normalize(-reflect(L, N));

	// calculate ambient term
	vec4 I_a = vec4(ambient);

	// calculate diffuse term
	vec4 I_d = vec4(diffuse) * max(dot(N, L), 0.0);
	I_d = clamp(I_d, 0.0, 1.0);

	// calculate specular term
	vec4 I_s = vec4(specular) * pow(max(dot(R, V),0.0), shininess);
	I_s = clamp(I_s, 0.0, 1.0);

	// compute total illumination
	gl_FragColor = I_a + I_d + I_s;
	gl_FragColor.a = 1.0; // always opaque
}