﻿/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: Phong.vert 537 2012-04-04 15:35:42Z cr333 $
*/
#version 120

// coordinates of vertices
uniform sampler2D coordinates;

// direction or position of light
uniform vec3 lightDirection;

// is light defined in world coordinates?
uniform bool lightWorldCoords;

// is light a position and not a direction?
uniform bool lightAbsoluteCoords;

// the following are all in viewing coordinates
varying vec3 N; // surface normal
varying vec3 v; // vertex position
varying vec3 l; // light position

varying float valid;


// forward declaration of mesh vertex shader functions (from Mesh.vert)
vec3 getVertex(sampler2D depthmap, vec2 position);
vec3 getVertexNormal(sampler2D depthmap, vec2 position, vec3 coords);

void main(void)  
{
	// vertex position and normal, in world coordinates
	vec3 vertex = getVertex(coordinates, gl_Vertex.xy);
	vec3 normal = getVertexNormal(coordinates, gl_Vertex.xy, vertex);

	// vertex position, in view coordinates
	v = vec3(gl_ModelViewMatrix * vec4(vertex, 1));

	// normal vector at vertex, in view coordinates
	N = normalize(gl_NormalMatrix * normal);

	// compute light direction, in viewing coordinates
	if(lightWorldCoords)
	{
		// translate if using absolute world coordinates
		if(lightAbsoluteCoords) l = lightDirection - vertex.xyz;
		else                    l = lightDirection;

		// transform to viewing coords
		l = gl_NormalMatrix * l;
	}
	else
	{
		// translate if using absolute view coordinates
		if(lightAbsoluteCoords) l = lightDirection - v;
		else                    l = lightDirection;
	}
	
	if(vertex.z > 4.5)       valid = 0.0; // saturated pixels
	else if(vertex.z < 0.01) valid = 0.0; // invalid pixels
	else                     valid = 1.0; // all other pixels

	// default transform of vertex position
	gl_Position = gl_ModelViewProjectionMatrix * vec4(vertex, 1);
}