/*
This source file is part of the RGBZ Video Camera project published in the paper

  Coherent Spatiotemporal Filtering, Upsampling and Rendering of RGBZ Videos
  Christian Richardt, Carsten Stoll, Neil A. Dodgson, Hans-Peter Seidel, Christian Theobalt
  Computer Graphics Forum (Proceedings of Eurographics), 31(2), May 2012
  <http://www.mpi-inf.mpg.de/resources/rgbz-camera/>

If you use our code, please cite our paper. Thanks.

Copyright (c) 2012 Christian Richardt <christian@richardt.name>


Code licence: GPL v3
----------------------------------------------------------------------------------------------------
This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, see <http://www.gnu.org/licenses/>.

$Id: SeparableBilateralY.frag 537 2012-04-04 15:35:42Z cr333 $
*/
#version 120
#extension GL_EXT_gpu_shader4 : enable

//---- Inputs -------------------------------------------------------------------------------------

// input texture
uniform sampler2D src;

uniform float sigmaR;
uniform float sigmaD;

//---- Main ---------------------------------------------------------------------------------------

void main()
{
	vec2 viewport = textureSize2D(src, 0);
	vec3 centre = texture2D(src, gl_FragCoord.xy / viewport).rgb;
	
	// accumulator for pixel values
	vec4 acc = vec4(centre, 1.0f);
	float sr2 = 1.0f / (2.0f * sigmaR * sigmaR);
	float sd2 = 1.0f / (2.0f * sigmaD * sigmaD);
	
	for(float dy = 1.0f; dy <= 2.0f * sigmaD; dy++)
	{
		vec3 col1 = texture2D(src, (gl_FragCoord.xy + vec2(0.0f, dy)) / viewport).rgb;
		vec3 col2 = texture2D(src, (gl_FragCoord.xy - vec2(0.0f, dy)) / viewport).rgb;
		
		float dr1 = distance(centre, col1);
		float dr2 = distance(centre, col2);
		float wd  = exp(-dy * dy * sd2);
		float wr1 = exp(-dr1 * dr1 * sr2);
		float wr2 = exp(-dr2 * dr2 * sr2);
		
		acc += wd * (wr1 * vec4(col1, 1) + wr2 * vec4(col2, 1));
	}

	gl_FragColor = vec4(acc.rgb / acc.a, 1);
}
