// LibAvWrap -- .NET wrapper for the libav* libraries
// --------------------------------------------------
// By Christian Richardt <christian@richardt.name>
// (based on AvDn 0.7, www.informikon.com/tutorials/)
//
// $Id: CodecContext.cpp 436 2010-10-15 16:26:38Z cr333 $

#include "stdafx.h"
#include "Frame.h"
#include "CodecContext.h"
#include "Packet.h"
#include "Stream.h"
#include "Utils.h"

namespace LibAvWrap
{
// constructors and destructor --------------------------------------------------------------------

// member functions -------------------------------------------------------------------------------

	//// cr333: <comment>
	//int AvDn::AvDnCodecContext::DecodeAudio(IntPtr samples, IntPtr in_ptr, int in_size )
	//{
	//	int out_size = 0;
	//	avcodec_decode_audio( pointer, (int16_t*)(void *)samples, &out_size, (uint8_t *)(void *)in_ptr, in_size);
	//	return out_size;
	//}
	//// cr333: </comment>

	// Decodes the packet payload into the frame.
	void LibAvCodecContext::DecodeVideo([Out]LibAvFrame^% f, LibAvPacket^ p, [Out]Int32% got_picture_ptr)
	{
		AVPacket* pPtr = p->Pointer;
		AVFrame*  fPtr = f->Pointer;

		// TODO: proper error handling
		int tmp = got_picture_ptr;
		int res = avcodec_decode_video2(pointer, fPtr, &tmp, pPtr);
		got_picture_ptr = tmp;

		// set size and format on the new frame
		f = gcnew LibAvFrame(fPtr, (LibAvWrap::PixelFormat)pointer->pix_fmt, pointer->width, pointer->height);
	}

	// Encodes samples into the given buffer location and size.
	int LibAvCodecContext::EncodeAudio(IntPtr buf_ptr, int size, IntPtr samples)
	{
		uint8_t* pBuffer  = (uint8_t*)(void*)buf_ptr;
		short*   pSamples = (short*)(void*)samples;
		// TODO: proper error handling
		return avcodec_encode_audio(pointer, pBuffer, size, pSamples);
	}

	// Encodes a frame into the given buffer location and size.
	int LibAvCodecContext::EncodeVideo(IntPtr buf_ptr, int size, LibAvFrame^ f)
	{
		// TODO: proper error handling
		return avcodec_encode_video(pointer, (uint8_t*)(void*)buf_ptr, size, f->Pointer);
	}

	// Sets the packet PKT_FLAG_KEY if the current coded_frame is a key frame.
	void LibAvCodecContext::HandlePacketFlagKey(LibAvPacket^ packet)
	{
		if(pointer->coded_frame->key_frame)
			packet->Pointer->flags |= PKT_FLAG_KEY;
	}

	// Finds and opens the decoder given the current CodecId.
	int LibAvCodecContext::OpenDecoder()
	{
		AVCodec* pCodec = avcodec_find_decoder(pointer->codec_id);
		// TODO: proper error handling
		return avcodec_open(pointer, pCodec);
	}

	// Finds and opens the encoder given the current CodecId.
	int LibAvCodecContext::OpenEncoder()
	{
		AVCodec* pCodec = avcodec_find_encoder(pointer->codec_id);
		if(!pCodec)
			throw gcnew Exception(String::Format(
				gcnew String("Could not find encoder for CodecID = {0}."), (LibAvWrap::CodecID)(int)(pointer->codec_id)));

		// TODO: proper error handling
		int result = avcodec_open(pointer, pCodec);
		LibAvUtils::ThrowIfError(result);
		return result;
	}

	// Sets the packet time stamp.
	void LibAvCodecContext::SetPacketPts(LibAvPacket^ p, LibAvStream^ s)
	{
		AVPacket* pPacket = p->Pointer;
		AVStream* st = s->Pointer;
		pPacket->pts = av_rescale_q(pointer->coded_frame->pts, pointer->time_base, st->time_base);
	}

// properties -------------------------------------------------------------------------------------

	// Setter for bit rate.
	void LibAvCodecContext::BitRate::set(int rate) { pointer->bit_rate = rate; }

	// Getter for frame width.
	int LibAvCodecContext::Width::get() { return pointer->width; }

	// Setter for frame width.
	void LibAvCodecContext::Width::set(int w) { pointer->width = w; }

	// Getter for frame height.
	int LibAvCodecContext::Height::get() { return pointer->height; }

	// Setter for frame height.
	void LibAvCodecContext::Height::set(int h) { pointer->height = h; }

	// Getter for number of pictures in a group of pictures.
	int LibAvCodecContext::GopSize::get() { return pointer->gop_size; }

	// Setter for number of pictures in a group of pictures.
	void LibAvCodecContext::GopSize::set(int s) { pointer->gop_size = s; }

	// Getter for pixel format.
	LibAvWrap::PixelFormat LibAvCodecContext::PixFmt::get() { return (LibAvWrap::PixelFormat)pointer->pix_fmt; }

	// Setter for pixel format.
	void LibAvCodecContext::PixFmt::set(LibAvWrap::PixelFormat f) { pointer->pix_fmt = (::PixelFormat)((int)f); }

	// Getter for macroblock decision mode.
	int LibAvCodecContext::MbDecision::get() { return pointer->mb_decision; }

	// Setter for macroblock decision mode.
	void LibAvCodecContext::MbDecision::set(int d) { pointer->mb_decision = d; }

	// Getter for samples per second.
	int LibAvCodecContext::SampleRate::get() { return pointer->sample_rate; }

	// Setter for samples per second.
	void LibAvCodecContext::SampleRate::set(int r) { pointer->sample_rate = r; }

	// Getter for number of audio channels.
	int LibAvCodecContext::Channels::get() { return pointer->channels; }

	// Setter for number of audio channels.
	void LibAvCodecContext::Channels::set(int c) { pointer->channels = c; }

	// Setter for codec type.
	void LibAvCodecContext::MediaType::set(LibAvWrap::MediaType t) { pointer->codec_type = (::AVMediaType)t; }

} // namespace LibAvWrap