// LibAvWrap -- .NET wrapper for the libav* libraries
// --------------------------------------------------
// By Christian Richardt <christian@richardt.name>
// (based on AvDn 0.7, www.informikon.com/tutorials/)
//
// $Id: Utils.cpp 436 2010-10-15 16:26:38Z cr333 $

#include "stdafx.h"
#include "ByteIOContext.h"
#include "FormatContext.h"
#include "OutputFormat.h"
#include "Utils.h"

namespace LibAvWrap
{
// constructors and destructor --------------------------------------------------------------------

// member functions -------------------------------------------------------------------------------

	// Guesses a video format from some given information.
	LibAvOutputFormat^ LibAvUtils::GuessType(String^ shortName, String^ filename, String^ mimeType)
	{
		// convert .NET strings to C strings
		IntPtr ipShortName = Marshal::StringToHGlobalAnsi(shortName);
		IntPtr ipFilename  = Marshal::StringToHGlobalAnsi(filename);
		IntPtr ipMimeType  = Marshal::StringToHGlobalAnsi(mimeType);
		char* strShortName = (char*)ipShortName.ToPointer();
		char* strFilename  = (char*)ipFilename.ToPointer();
		char* strMimeType  = (char*)ipMimeType.ToPointer();

		// call to native function
		AVOutputFormat* ptr = guess_format(
			strlen(strShortName) > 0 ? strShortName : NULL,
			strlen(strFilename) > 0 ? strFilename : NULL,
			strlen(strMimeType) > 0 ? strMimeType : NULL);

		// clean up string pointer
		Marshal::FreeHGlobal(ipShortName);
		Marshal::FreeHGlobal(ipFilename);
		Marshal::FreeHGlobal(ipMimeType);

		// return the created AVOutputFormat
		return gcnew LibAvOutputFormat(ptr);
	}

	void LibAvUtils::UrlFClose(LibAvByteIOContext^ bioc)
	{
		url_fclose(bioc->Pointer);
	}

	/*
	cr333 (2009-03-07):
		This used to be the code that wraps url_fopen in the obvious way, but the pointers break
		somewhere in the middle section, causing havoc all over the place.

	int AvDnUtils::UrlFOpen(AvDnByteIOContext^ bioc, System::String ^filename, int flag) 
	{
		IntPtr ip = Marshal::StringToHGlobalAnsi(filename);
		char* str = (char *)ip.ToPointer();

		// the following (lines 1 & 3) are a hack to work around pointer-to-pointer issues
		ByteIOContext* pbioc = bioc->NativePtr;
		int ret = url_fopen(&pbioc, str, flag);
		bioc->NativePtr = pbioc;

		Marshal::FreeHGlobal(ip);
		return ret;
	}
	*/

	int LibAvWrap::LibAvUtils::UrlFOpen(LibAvFormatContext^ fc, String^ filename, int flag)
	{
		// translate String to char*
		IntPtr ip = Marshal::StringToHGlobalAnsi(filename);
		char* strFilename = (char*)ip.ToPointer();

		// call native function
		int ret = url_fopen(&(fc->Pointer->pb), strFilename, URL_WRONLY);

		// clean up string memory
		Marshal::FreeHGlobal(ip);
		return ret;
	}

// properties -------------------------------------------------------------------------------------

} // namespace LibAvWrap