﻿// $Id: UndistortFilter.cs 547 2012-01-11 16:04:00Z cr333 $
using System.Collections.Generic;
using System.ComponentModel;
using OpenTK;
using VideoLib.Vision;

namespace VideoLib.OpenGL
{
	/// <summary>Node that compensates for radial and tangential lens distortion.</summary>
	public class UndistortFilter : FboNode
	{
		/// <summary>Gets or sets the intrinsic camera parameters used for undistortion.</summary>
		[Description("The intrinsic camera parameters used for undistortion.")]
		public IntrinsicParameters Parameters { get; set; }


		// the following private properties are passed through to the shader

		[Uniform()]
		private Vector2 fc { get { return new Vector2(Parameters.Fx, Parameters.Fy); } }

		[Uniform()]
		private Vector2 cc { get { return new Vector2(Parameters.Cx, Parameters.Cy); } }

		[Uniform()]
		private float ac { get { return Parameters.Alpha; } }

		[Uniform()]
		private float[] kc { get { return Parameters.K; } }


		/// <summary>Initialises a new instance of the <see cref="UndistortFilter"/> class.</summary>
		public UndistortFilter()
		{
			AddDefaultPins();
		}

		/// <summary>Initialises the OpenGL state.</summary>
		public override void Init()
		{
			// load, compile and link shaders
			programID = Utils.LinkShaders(
				Utils.CreateShaderFromResource("Shaders.UndistortFilter.frag")
			);

			// map textures
			textureMap = new List<TextureBinding> {
				new TextureBinding { Pin = "in", Uniform = "input", Unit = 0 },
			};

			base.Init();
		}

		/// <summary>Draws to the framebuffer object using the given input textures.</summary>
		/// <param name="inputs">The input textures.</param>
		/// <returns>The resulting texture.</returns>
		public override TextureFrame Draw(Dictionary<string, TextureFrame> inputs)
		{
			// allocate a texture that is identical to the input
			var resultTexture = TextureFrame.CreateToMatch(inputs["in"]);
			resultTexture.Alloc();

			Draw(inputs, resultTexture.Texture);

			return resultTexture;
		}
	}
}
