﻿// $Id: FlowMorpher.frag 458 2010-10-25 11:16:45Z cr333 $
#version 120
#extension GL_EXT_gpu_shader4 : enable

/*
This source code was part of the Floating Textures test program, which
is a derivative of the complete Floating Textures program containing
the offline version with prereconstructed geometry and prerecorded
video. 
For more information on Floating Textures please have a look at the article

"Floating Textures" 
by M. Eisemann, B. De Decker, M. Magnor, 
P. Bekaert, E. de Aguiar, N. Ahmed, C. Theobalt, A. Sellent
Computer Graphics Forum (Proc. Eurographics EG'08), vol. 27, no. 2, April 2008 


Copyright (C) 2008 Martin Eisemann

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 3 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT 
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, see <http://www.gnu.org/licenses/>.
*/
uniform sampler2D source;
uniform sampler2D target;
uniform sampler2D flow;

// This is the time value [0..1]. 0: source, 1: target
uniform float value;

void main()
{
	// original coordinates //
	vec4 tC = vec4(gl_FragCoord.xy, gl_FragCoord.xy);

	// calculate the warped coordinates, depending on the flow field //
	vec4 flows = texture2D(flow, tC.xy / textureSize2D(flow, 0)); // xy is the warp from S to T, zw from T to S
	vec4 warpTC = tC - vec4(value, -value, 1.0f - value, -(1.0f - value)) * flows;

	vec4 colorSource = texture2D(source, warpTC.xy / textureSize2D(flow, 0));
	vec4 colorTarget = texture2D(target, warpTC.zw / textureSize2D(flow, 0));

	gl_FragColor = mix(colorSource, colorTarget, clamp(value, 0.0, 1.0));
}